type Metadata = Record<string, unknown>;

type BlobInput = ReadableStream | string | ArrayBuffer | Blob;
type Fetcher = typeof globalThis.fetch;
declare enum HTTPMethod {
    DELETE = "delete",
    GET = "get",
    PUT = "put"
}

interface MakeStoreRequestOptions {
    body?: BlobInput | null;
    headers?: Record<string, string>;
    key: string;
    metadata?: Metadata;
    method: HTTPMethod;
    storeName: string;
}
interface ClientOptions {
    apiURL?: string;
    edgeURL?: string;
    fetch?: Fetcher;
    siteID: string;
    token: string;
}
declare class Client {
    private apiURL?;
    private edgeURL?;
    private fetch?;
    private siteID;
    private token;
    constructor({ apiURL, edgeURL, fetch, siteID, token }: ClientOptions);
    private getFinalRequest;
    makeRequest({ body, headers: extraHeaders, key, metadata, method, storeName }: MakeStoreRequestOptions): Promise<Response | null>;
}

interface BaseStoreOptions {
    client: Client;
}
interface DeployStoreOptions extends BaseStoreOptions {
    deployID: string;
}
interface NamedStoreOptions extends BaseStoreOptions {
    name: string;
}
type StoreOptions = DeployStoreOptions | NamedStoreOptions;
interface SetOptions {
    /**
     * Arbitrary metadata object to associate with an entry. Must be seralizable
     * to JSON.
     */
    metadata?: Metadata;
}
type BlobWithMetadata = {
    etag?: string;
} & {
    metadata: Metadata;
};
declare class Store {
    private client;
    private name;
    constructor(options: StoreOptions);
    delete(key: string): Promise<void>;
    get(key: string): Promise<string>;
    get(key: string, { type }: {
        type: 'arrayBuffer';
    }): Promise<ArrayBuffer>;
    get(key: string, { type }: {
        type: 'blob';
    }): Promise<Blob>;
    get(key: string, { type }: {
        type: 'json';
    }): Promise<any>;
    get(key: string, { type }: {
        type: 'stream';
    }): Promise<ReadableStream>;
    get(key: string, { type }: {
        type: 'text';
    }): Promise<string>;
    getWithMetadata(key: string): Promise<{
        data: string;
    } & BlobWithMetadata>;
    getWithMetadata(key: string, { type }: {
        type: 'arrayBuffer';
    }): Promise<{
        data: ArrayBuffer;
    } & BlobWithMetadata>;
    getWithMetadata(key: string, { type }: {
        type: 'blob';
    }): Promise<{
        data: Blob;
    } & BlobWithMetadata>;
    getWithMetadata(key: string, { type }: {
        type: 'json';
    }): Promise<{
        data: any;
    } & BlobWithMetadata>;
    getWithMetadata(key: string, { type }: {
        type: 'stream';
    }): Promise<{
        data: ReadableStream;
    } & BlobWithMetadata>;
    getWithMetadata(key: string, { type }: {
        type: 'text';
    }): Promise<{
        data: string;
    } & BlobWithMetadata>;
    set(key: string, data: BlobInput, { metadata }?: SetOptions): Promise<void>;
    setJSON(key: string, data: unknown, { metadata }?: SetOptions): Promise<void>;
}

interface GetDeployStoreOptions extends Partial<ClientOptions> {
    deployID?: string;
}
/**
 * Gets a reference to a deploy-scoped store.
 */
declare const getDeployStore: (options?: GetDeployStoreOptions) => Store;
interface GetStoreOptions extends Partial<ClientOptions> {
    deployID?: string;
    name?: string;
}
/**
 * Gets a reference to a store.
 *
 * @param input Either a string containing the store name or an options object
 */
declare const getStore: {
    (name: string): Store;
    (options: GetStoreOptions): Store;
};

export { getDeployStore, getStore };
