// src/environment.ts
import { Buffer } from "buffer";
import { env } from "process";
var NETLIFY_CONTEXT_VARIABLE = "NETLIFY_BLOBS_CONTEXT";
var getEnvironmentContext = () => {
  if (!env[NETLIFY_CONTEXT_VARIABLE]) {
    return {};
  }
  const data = Buffer.from(env[NETLIFY_CONTEXT_VARIABLE], "base64").toString();
  try {
    return JSON.parse(data);
  } catch {
  }
  return {};
};
var MissingBlobsEnvironmentError = class extends Error {
  constructor(requiredProperties) {
    super(
      `The environment has not been configured to use Netlify Blobs. To use it manually, supply the following properties when creating a store: ${requiredProperties.join(
        ", "
      )}`
    );
    this.name = "MissingBlobsEnvironmentError";
  }
};

// src/metadata.ts
import { Buffer as Buffer2 } from "buffer";
var BASE64_PREFIX = "b64;";
var METADATA_HEADER_INTERNAL = "x-amz-meta-user";
var METADATA_HEADER_EXTERNAL = "netlify-blobs-metadata";
var METADATA_MAX_SIZE = 2 * 1024;
var encodeMetadata = (metadata) => {
  if (!metadata) {
    return null;
  }
  const encodedObject = Buffer2.from(JSON.stringify(metadata)).toString("base64");
  const payload = `b64;${encodedObject}`;
  if (METADATA_HEADER_EXTERNAL.length + payload.length > METADATA_MAX_SIZE) {
    throw new Error("Metadata object exceeds the maximum size");
  }
  return payload;
};
var decodeMetadata = (headers) => {
  if (!headers) {
    return {};
  }
  const metadataHeader = headers.get(METADATA_HEADER_INTERNAL);
  if (!metadataHeader || !metadataHeader.startsWith(BASE64_PREFIX)) {
    return {};
  }
  const encodedData = metadataHeader.slice(BASE64_PREFIX.length);
  const decodedData = Buffer2.from(encodedData, "base64").toString();
  const metadata = JSON.parse(decodedData);
  return metadata;
};

// src/retry.ts
var DEFAULT_RETRY_DELAY = 5e3;
var MIN_RETRY_DELAY = 1e3;
var MAX_RETRY = 5;
var RATE_LIMIT_HEADER = "X-RateLimit-Reset";
var fetchAndRetry = async (fetch, url, options, attemptsLeft = MAX_RETRY) => {
  try {
    const res = await fetch(url, options);
    if (attemptsLeft > 0 && (res.status === 429 || res.status >= 500)) {
      const delay = getDelay(res.headers.get(RATE_LIMIT_HEADER));
      await sleep(delay);
      return fetchAndRetry(fetch, url, options, attemptsLeft - 1);
    }
    return res;
  } catch (error) {
    if (attemptsLeft === 0) {
      throw error;
    }
    const delay = getDelay();
    await sleep(delay);
    return fetchAndRetry(fetch, url, options, attemptsLeft - 1);
  }
};
var getDelay = (rateLimitReset) => {
  if (!rateLimitReset) {
    return DEFAULT_RETRY_DELAY;
  }
  return Math.max(Number(rateLimitReset) * 1e3 - Date.now(), MIN_RETRY_DELAY);
};
var sleep = (ms) => new Promise((resolve) => {
  setTimeout(resolve, ms);
});

// src/client.ts
var Client = class {
  constructor({ apiURL, edgeURL, fetch, siteID, token }) {
    this.apiURL = apiURL;
    this.edgeURL = edgeURL;
    this.fetch = fetch;
    this.siteID = siteID;
    this.token = token;
  }
  async getFinalRequest(storeName, key, method, metadata) {
    const encodedKey = encodeURIComponent(key);
    const encodedMetadata = encodeMetadata(metadata);
    if (this.edgeURL) {
      const headers = {
        authorization: `Bearer ${this.token}`
      };
      if (encodedMetadata) {
        headers[METADATA_HEADER_EXTERNAL] = encodedMetadata;
      }
      return {
        headers,
        url: `${this.edgeURL}/${this.siteID}/${storeName}/${encodedKey}`
      };
    }
    const apiURL = `${this.apiURL ?? "https://api.netlify.com"}/api/v1/sites/${this.siteID}/blobs/${encodedKey}?context=${storeName}`;
    const apiHeaders = { authorization: `Bearer ${this.token}` };
    if (encodedMetadata) {
      apiHeaders[METADATA_HEADER_EXTERNAL] = encodedMetadata;
    }
    const fetch = this.fetch ?? globalThis.fetch;
    const res = await fetch(apiURL, { headers: apiHeaders, method });
    if (res.status !== 200) {
      throw new Error(`${method} operation has failed: API returned a ${res.status} response`);
    }
    const { url } = await res.json();
    const userHeaders = encodedMetadata ? { [METADATA_HEADER_INTERNAL]: encodedMetadata } : void 0;
    return {
      headers: userHeaders,
      url
    };
  }
  async makeRequest({ body, headers: extraHeaders, key, metadata, method, storeName }) {
    const { headers: baseHeaders = {}, url } = await this.getFinalRequest(storeName, key, method, metadata);
    const headers = {
      ...baseHeaders,
      ...extraHeaders
    };
    if (method === "put" /* PUT */) {
      headers["cache-control"] = "max-age=0, stale-while-revalidate=60";
    }
    const options = {
      body,
      headers,
      method
    };
    if (body instanceof ReadableStream) {
      options.duplex = "half";
    }
    const fetch = this.fetch ?? globalThis.fetch;
    const res = await fetchAndRetry(fetch, url, options);
    if (res.status === 404 && method === "get" /* GET */) {
      return null;
    }
    if (res.status !== 200) {
      throw new Error(`${method} operation has failed: store returned a ${res.status} response`);
    }
    return res;
  }
};
var getClientOptions = (options, contextOverride) => {
  const context = contextOverride ?? getEnvironmentContext();
  const siteID = context.siteID ?? options.siteID;
  const token = context.token ?? options.token;
  if (!siteID || !token) {
    throw new MissingBlobsEnvironmentError(["siteID", "token"]);
  }
  const clientOptions = {
    apiURL: context.apiURL ?? options.apiURL,
    edgeURL: context.edgeURL ?? options.edgeURL,
    fetch: options.fetch,
    siteID,
    token
  };
  return clientOptions;
};

// src/store.ts
var Store = class {
  constructor(options) {
    this.client = options.client;
    this.name = "deployID" in options ? `deploy:${options.deployID}` : options.name;
  }
  async delete(key) {
    await this.client.makeRequest({ key, method: "delete" /* DELETE */, storeName: this.name });
  }
  async get(key, options) {
    const { type } = options ?? {};
    const res = await this.client.makeRequest({ key, method: "get" /* GET */, storeName: this.name });
    if (res === null) {
      return res;
    }
    if (type === void 0 || type === "text") {
      return res.text();
    }
    if (type === "arrayBuffer") {
      return res.arrayBuffer();
    }
    if (type === "blob") {
      return res.blob();
    }
    if (type === "json") {
      return res.json();
    }
    if (type === "stream") {
      return res.body;
    }
    throw new Error(`Invalid 'type' property: ${type}. Expected: arrayBuffer, blob, json, stream, or text.`);
  }
  async getWithMetadata(key, options) {
    const { type } = options ?? {};
    const res = await this.client.makeRequest({ key, method: "get" /* GET */, storeName: this.name });
    const etag = res?.headers.get("etag") ?? void 0;
    let metadata = {};
    try {
      metadata = decodeMetadata(res?.headers);
    } catch {
      throw new Error(
        "An internal error occurred while trying to retrieve the metadata for an entry. Please try updating to the latest version of the Netlify Blobs client."
      );
    }
    if (res === null) {
      return null;
    }
    if (type === void 0 || type === "text") {
      return { data: await res.text(), etag, metadata };
    }
    if (type === "arrayBuffer") {
      return { data: await res.arrayBuffer(), etag, metadata };
    }
    if (type === "blob") {
      return { data: await res.blob(), etag, metadata };
    }
    if (type === "json") {
      return { data: await res.json(), etag, metadata };
    }
    if (type === "stream") {
      return { data: res.body, etag, metadata };
    }
    throw new Error(`Invalid 'type' property: ${type}. Expected: arrayBuffer, blob, json, stream, or text.`);
  }
  async set(key, data, { metadata } = {}) {
    await this.client.makeRequest({
      body: data,
      key,
      metadata,
      method: "put" /* PUT */,
      storeName: this.name
    });
  }
  async setJSON(key, data, { metadata } = {}) {
    const payload = JSON.stringify(data);
    const headers = {
      "content-type": "application/json"
    };
    await this.client.makeRequest({
      body: payload,
      headers,
      key,
      metadata,
      method: "put" /* PUT */,
      storeName: this.name
    });
  }
};

// src/store_factory.ts
var getDeployStore = (options = {}) => {
  const context = getEnvironmentContext();
  const deployID = options.deployID ?? context.deployID;
  if (!deployID) {
    throw new MissingBlobsEnvironmentError(["deployID"]);
  }
  const clientOptions = getClientOptions(options, context);
  const client = new Client(clientOptions);
  return new Store({ client, deployID });
};
var getStore = (input) => {
  if (typeof input === "string") {
    const clientOptions = getClientOptions({});
    const client = new Client(clientOptions);
    return new Store({ client, name: input });
  }
  if (typeof input?.name === "string") {
    const { name } = input;
    const clientOptions = getClientOptions(input);
    if (!name) {
      throw new MissingBlobsEnvironmentError(["name"]);
    }
    const client = new Client(clientOptions);
    return new Store({ client, name });
  }
  if (typeof input?.deployID === "string") {
    const clientOptions = getClientOptions(input);
    const { deployID } = input;
    if (!deployID) {
      throw new MissingBlobsEnvironmentError(["deployID"]);
    }
    const client = new Client(clientOptions);
    return new Store({ client, deployID });
  }
  throw new Error(
    "The `getStore` method requires the name of the store as a string or as the `name` property of an options object"
  );
};
export {
  getDeployStore,
  getStore
};
